package data_manage

import (
	sql2 "database/sql"
	"eta/eta_task/global"
	"eta/eta_task/utils"
	"time"
)

type BaseFromTradeEicIndexV2 struct {
	BaseFromEicIndexId     int `gorm:"column:base_from_eic_index_id;primaryKey;autoIncrement"`
	Type                   string
	EicCode                string
	Name                   string
	Status                 string
	GasDayStart            string
	GasInStorage           string
	GasInStorageCode       string
	Consumption            string
	ConsumptionCode        string
	ConsumptionFull        string
	ConsumptionFullCode    string
	Full                   string
	FullCode               string
	Trend                  string
	TrendCode              string
	Injection              string
	InjectionCode          string
	Withdrawal             string
	WithdrawalCode         string
	WorkingGasVolume       string
	WorkingGasVolumeCode   string
	InjectionCapacity      string
	InjectionCapacityCode  string
	WithdrawalCapacity     string
	WithdrawalCapacityCode string
	Info                   string
	Parent                 string
	CreateTime             time.Time
	ModifyTime             time.Time
}

func GetBaseFromEicIndexMaxDate() (maxDate time.Time, err error) {
	o := global.DbMap[utils.DbNameIndex]
	sql := ` SELECT max(a.gas_day_start) as max_date FROM base_from_trade_eic_index_v2 as a `
	var timeNull sql2.NullTime
	err = o.Raw(sql).Scan(&timeNull).Error
	if err != nil {
		return
	}
	if timeNull.Valid {
		maxDate = timeNull.Time
	}
	return
}

func GetBaseFromEicIndexAll(dateStr string) (list []*BaseFromTradeEicIndexV2, err error) {
	o := global.DbMap[utils.DbNameIndex]
	sql := `SELECT * FROM base_from_trade_eic_index_v2 where gas_day_start >=?`
	err = o.Raw(sql, dateStr).Find(&list).Error
	return
}

func AddBaseFromEicIndexV2(item *BaseFromTradeEicIndexV2) (lastId int64, err error) {
	err = global.DbMap[utils.DbNameIndex].Create(item).Error
	if err != nil {
		return
	}
	lastId = int64(item.BaseFromEicIndexId)
	return
}

type EicIndexResp struct {
	Ret     int
	Msg     string
	ErrMsg  string
	ErrCode string
	Data    []*BaseFromTradeEicIndexV2
}
