package data_manage

import (
	"eta/eta_task/global"
	"eta/eta_task/utils"
)

type BaseFromSmmIndexItem struct {
	BaseFromSmmIndexId int32   `gorm:"column:base_from_smm_index_id;primaryKey;autoIncrement" json:"base_from_smm_index_id"`
	ClassifyId         int32   `json:"classify_id"` // Smm原始数据指标分类id
	Interface          string  `json:"interface"`
	Name               string  `json:"name"`
	IndexCode          string  `json:"index_code"`           // 指标编码
	IndexName          string  `json:"index_name"`           // 指标名称
	Type1              string  `gorm:"type_1" json:"type_1"` // 类型1
	Type2              string  `gorm:"type_2" json:"type_2"` // 类型2
	Type3              string  `gorm:"type_3" json:"type_3"` // 类型3
	Frequency          string  `json:"frequency"`            // 频度
	Unit               string  `json:"unit"`                 // 单位
	ApiStartTime       string  `json:"api_start_time"`       // 接口开始日期
	ApiUpdateTime      string  `json:"api_update_time"`      // 接口更新日期
	StartTime          string  `json:"start_time"`           // 开始日期
	FinishTime         string  `json:"finish_time"`          // 完成日期
	CreateTime         string  `json:"create_time"`
	ModifyTime         string  `json:"modify_time"`
	StartDate          string  `json:"start_date"`
	EndDate            string  `json:"end_date"`
	IsGet              int32   `json:"is_get"`           // 0:未获取数据,1:已获取数据
	Sort               int32   `json:"sort"`             // 排序
	BaseFileName       string  `json:"base_file_name"`   // 文件目录
	RenameFileName     string  `json:"rename_file_name"` // 重命名文件目录
	TerminalCode       string  `json:"terminal_code"`    // 所属终端编码
	DataState          string  `json:"data_state"`       // 指标数据状态 normal/空白-正常更新 ceased-已停更irregular-更新依赖外部会有不定期延迟
	ReleaseTime        string  `json:"release_time"`     // 预计发布时间
	IsStop             int32   `json:"is_stop"`          // 是否停更:1:停更,0:未停更
	EndValue           float64 `json:"end_value"`        // 指标的最新值
}

func (m *BaseFromSmmIndexItem) ConvertTimeStr() {
	m.StartDate = utils.GormDateStrToDateStr(m.StartDate)
	m.EndDate = utils.GormDateStrToDateStr(m.EndDate)
	m.StartTime = utils.GormDateStrToDateStr(m.StartTime)
	m.FinishTime = utils.GormDateStrToDateStr(m.FinishTime)
	m.CreateTime = utils.GormDateStrToDateStr(m.CreateTime)
	m.ModifyTime = utils.GormDateStrToDateTimeStr(m.ModifyTime)
	return
}

// GetBaseFromSmmIndexItemItemByCode
// @Description: 根据指标编码获取指标信息
// @author: Roc
// @datetime 2024-03-11 16:41:02
// @param source int
// @param edbCode string
// @return item *BaseFromSmmIndexItem
// @return err error
func GetBaseFromSmmIndexItemItemByCode(edbCode string) (item *BaseFromSmmIndexItem, err error) {
	o := global.DbMap[utils.DbNameIndex]
	sql := ` SELECT * FROM base_from_smm_index WHERE index_code=? `
	err = o.Raw(sql, edbCode).First(&item).Error
	if err != nil {
		return
	}
	if item != nil && item.BaseFromSmmIndexId > 0 {
		item.ConvertTimeStr()
	}
	return
}
