package edb_refresh

import (
	"errors"
	"eta/eta_task/global"
	"eta/eta_task/utils"
	"time"
)

// EdbRefreshConfig
// @Description: 指标的刷新时间配置表
type EdbRefreshConfig struct {
	EdbRefreshConfigId  int       `gorm:"column:edb_refresh_config_id;primaryKey;autoIncrement"`
	RefreshFrequency    string    `description:"刷新频率"`
	RefreshFrequencyDay int       `description:"具体刷新的日期"`
	RefreshTime         string    `description:"刷新时间"`
	RefreshAllData      int       `description:"是否刷新所有数据，0：否，1：刷新所有数据"`
	RefreshDataNum      int       `description:"刷新单元格数"`
	ModifyTime          time.Time `description:"最晚一次的更新时间"`
	CreateTime          time.Time `description:"添加时间"`
}

// Add
// @Description: 添加
// @author: Roc
// @receiver m
// @datetime 2023-12-14 16:11:10
// @param cols []string
// @return err error
func (m *EdbRefreshConfig) Add() (err error) {
	if m.EdbRefreshConfigId > 0 {
		err = errors.New("该配置已存在")
		return
	}
	err = global.DbMap[utils.DbNameIndex].Create(m).Error
	return
}

// Update
// @Description: 更新
// @author: Roc
// @receiver m
// @datetime 2023-12-14 16:11:10
// @param cols []string
// @return err error
func (m *EdbRefreshConfig) Update(cols []string) (err error) {
	err = global.DbMap[utils.DbNameIndex].Select(cols).Updates(m).Error
	return
}

// Delete
// @Description: 删除
// @author: Roc
// @receiver m
// @datetime 2023-12-14 16:11:10
// @return err error
func (m *EdbRefreshConfig) Delete() (err error) {
	err = global.DbMap[utils.DbNameIndex].Delete(m).Error
	return
}

// GetEdbRefreshConfigListByCondition
// @Description: 根据条条件获取刷新配置列表
// @author: Roc
// @datetime 2024-01-09 13:28:49
// @param condition string
// @param pars []interface{}
// @return list []*EdbRefreshDefaultConfig
// @return err error
func GetEdbRefreshConfigListByCondition(condition string, pars []interface{}) (list []*EdbRefreshConfig, err error) {
	o := global.DbMap[utils.DbNameIndex]
	sql := `SELECT * FROM edb_refresh_config
         WHERE 1 = 1 `

	if condition != "" {
		sql += condition
	}
	sql += ` ORDER BY edb_refresh_config_id ASC `
	err = o.Raw(sql, pars...).Find(&list).Error
	return
}
