package data_manage

import (
	sql2 "database/sql"
	"eta/eta_task/global"
	"eta/eta_task/utils"
	"time"
)

// BaseFromChangesVisitorsCovid 谷歌出行指数
type BaseFromChangesVisitorsCovid struct {
	Id                  uint64    `gorm:"column:id;primaryKey;autoIncrement"` //序号
	Entity              string    `gorm:"column:entity"`                      //国家
	Code                string    `gorm:"column:code"`                        //国家编码
	EdbCode             string    `gorm:"column:edb_code"`                    //指标ID
	Day                 time.Time `gorm:"column:day"`                         //统计的日期
	RetailAndRecreation string    `gorm:"column:retail_and_recreation"`
	GroceryAndPharmacy  string    `gorm:"column:grocery_and_pharmacy"`
	Residential         string    `gorm:"column:residential"`
	TransitStations     string    `gorm:"column:transit_stations"`
	Parks               string    `gorm:"column:parks"`
	Workplaces          string    `gorm:"column:workplaces"`
	Total               string    `gorm:"column:total"`
	CreateTime          time.Time `gorm:"column:create_time"` //创建时间
	ModifyTime          time.Time `gorm:"column:modify_time"` //修改时间
}

// AddBaseFromChangesVisitorsCovid 批量添加谷歌出行记录
func AddBaseFromChangesVisitorsCovid(item *BaseFromChangesVisitorsCovid) (err error) {
	err = global.DbMap[utils.DbNameIndex].Create(item).Error
	return
}

// GetLatestBaseFromChangesVisitorsCovid 查询最新的日期
func GetLatestBaseFromChangesVisitorsCovid() (item *BaseFromChangesVisitorsCovid, err error) {
	o := global.DbMap[utils.DbNameIndex]
	sql := `SELECT * FROM base_from_changes_visitors_covid order by day desc, id desc limit 1`
	err = o.Raw(sql).First(&item).Error
	return
}

// GetBaseFromChangesVisitorsCovidByEntityDay 查询国家和统计日期查询记录是否存在
func GetBaseFromChangesVisitorsCovidByEntityDay(entity, day string) (item *BaseFromChangesVisitorsCovid, err error) {
	o := global.DbMap[utils.DbNameIndex]
	sql := `SELECT * FROM base_from_changes_visitors_covid where entity=? and day=? limit 1`
	err = o.Raw(sql, entity, day).First(&item).Error
	return
}

type ChangesVisitorsCovidIndexResp struct {
	Ret     int
	Msg     string
	ErrMsg  string
	ErrCode string
	Data    []*BaseFromChangesVisitorsCovid
}

func GetChangesVisitorsCovidAll(dateTime string) (items []*BaseFromChangesVisitorsCovid, err error) {
	o := global.DbMap[utils.DbNameIndex]
	sql := `SELECT * FROM base_from_changes_visitors_covid WHERE day>=? `
	err = o.Raw(sql, dateTime).Find(&items).Error
	return
}

func GetChangesVisitorsCovidMaxDate() (maxDate time.Time, err error) {
	o := global.DbMap[utils.DbNameIndex]
	sql := ` SELECT MAX(a.day) AS max_date FROM base_from_changes_visitors_covid AS a `
	var timeNull sql2.NullTime
	err = o.Raw(sql).Scan(&timeNull).Error
	if err != nil {
		return
	}
	if timeNull.Valid {
		maxDate = timeNull.Time
	}
	return
}
